<?php

/**
 * @file
 */
class imagefield_crop_image_convert {
  private $table_groups = array(
    'data' => 'FIELD_LOAD_CURRENT',
    'revision' => 'FIELD_LOAD_REVISION'
  );
  private $imagefield_crop_widget = array(
    'type' => 'imagefield_crop_widget',
    'module' => 'imagefield_crop',
    'weight' => '8',
    'settings' => array()
  );
  private $image_widget = array(
    'type' => 'image_image',
    'module' => 'image',
    'weight' => '14',
    'active' => 1,
    'settings' => array(
      'progress_indicator' => 'throbber',
      'preview_image_style' => 'thumbnail',
    )
  );
  private $image_teaser = array(
    'label' => 'hidden',
    'type' => 'image',
    'weight' => '1',
    'settings' => array(
      'image_style' => '',
      'image_link' => '',
    ),
    'module' => 'image',
  );

  private $mediafront = array(
    'field_type' => '0',
    'media_type' => 'media',
    'preview' => '0',
    'thumbnail' => '0',
    'custom' => '',
  );

  function __construct() {
    watchdog('imagefield_crop_image_convert', 'construct', 'UI loaded');
  }

  /**
   * Return form to drupal_get_form in menu.
   */
  function admin_settings($form, $form_state) {
    $form = array();
    $form['#attached']['css'][] = drupal_get_path('module', 'imagefield_crop_image_convert') . '/imagefield_crop_image_convert.css';
    $input = false;
    if (!empty($form_state['input'])) {
      $input = $form_state['input'];
    }
    $field_map = field_info_field_map();
    $image_fields = array(''=>t('<none>'));
    foreach($field_map as $k => $v){
      if($v['type']=='image'){
        $image_fields[t('Convertable')][$k] = $k;
      }
      if($v['type']=='imagefield_crop'){
        $image_fields[t('Unconvertable')][$k] = $k;
      }
    }
    $form['field_to_convert'] = array(
      '#type' => 'select',
      '#options' => $image_fields,
      '#title' => t('Available Image Field(s)'),
      '#default_value' => ($input==FALSE ? NULL : $input['field_to_convert']),
    );

    $form['action_type'] = array(
      '#type' => 'select',
      '#options' => array(''=>t('Select Action'), 'analyze' => t('Analyze field'), 'convert' => t('Convert field'), 'unconvert' => t('Unconvert field')),
      '#title' => t('Action to take'),
      '#default_value' =>($input==FALSE ? NULL : $input['action_type']),
      '#required' => true,
    );

    $form['field_settings']= array(
      '#type'=>'fieldset',
      '#title'=> t('Image Field Settings'),
      '#states' => array(
        'visible' => array(
          array('select[name="action_type"]' => array('value' => 'convert')),
          'OR',
          array('select[name="action_type"]' => array('value' => 'unconvert')),
        ),
      ),
    );

    $form['field_settings']['progress_indicator'] = array(
      '#type' => 'select',
      '#title' => t('Progress Indicator'),
      '#options' => array(
        'throbber'=>t('throbber'),
        'bar'=>t('bar'),
        'none'=>t('none'),
      ),
      '#default_value' => $input!==FALSE ? $input['progress_indicator'] : 'throbber',
    );
    $styles = image_styles();
    $styles_list =array();
    foreach($styles as $k=>$v){
      $styles_list[$k] = $v['label'];
    }
    $form['field_settings']['preview_image_style'] = array(
      '#type' => 'select',
      '#options' => array(''=>t('<no preview>')) + $styles_list,
      '#title' => t('Preview image style'),
      '#default_value' => $input!==FALSE ? $input['preview_image_style'] : 'thumbnail',
    );

    $form['crop_widget_settings']= array(
      '#type'=>'fieldset',
      '#title'=> t('Imagefield Crop Settings'),
      '#states' => array(
        'visible' => array(
          'select[name="action_type"]' => array('value' => 'convert'),
        ),
      ),
    );

    $form['crop_widget_settings']['collapsible'] = array(
      '#type'=>'radios',
      '#title'=>t('Collapsible behavior'),
      '#options'=>array(
        '1'=>t('None.'),
        '2'=>t('Collapsible, expanded by default.'),
        '3'=>t('Collapsible, collapsed by default.'),
       ),
       '#default_value' => $input!==FALSE ? $input['collapsible'] : 2,
    );

    $form['crop_widget_settings']['resolution'] = array(
      '#type'=>'fieldset',
      '#title'=>t('The resolution to crop the image onto'),
      '#attributes'=> array(
        'class' => array('group-box')
      ),
    );
    $form['crop_widget_settings']['resolution']['resolution_x'] = array(
      '#type'=>'textfield',
      '#title'=>t('Width'),
      '#size'=>'6',
      '#field_suffix'=>'&nbsp;×&nbsp;&nbsp;',
    );
    $form['crop_widget_settings']['resolution']['resolution_y'] = array(
      '#type'=>'textfield',
      '#title'=>t('Height'),
      '#size'=>'6',
      '#field_suffix'=>'px',
    );


    $form['crop_widget_settings']['resolution']['desc'] = array(
      '#type'=>'markup',
      '#markup' =>'<div class="clearfix"></div>'.t('The output resolution of the cropped image, expressed as WIDTHxHEIGHT (e.g. 640x480). Leave blank to skip rescale after cropping. Note: output resolution must be defined in order to present a dynamic preview.'),
    );



    $form['crop_widget_settings']['validate_resolution'] = array(
      '#type'=>'checkbox',
      '#title'=>t('Validate minimum resolution'),
      '#description'=> t("If output resolution is set, but validation is disabled then images with smaller dimensions won't be cropped."),
      '#default_value' => ($input!==FALSE && isset($input['validate_resolution'])) ? $input['validate_resolution'] : false,
    );

    $form['crop_widget_settings']['select_maximum_area'] = array(
      '#type'=>'checkbox',
      '#title'=>t('Select maximum possible area'),
      '#description'=> t("If checked, widget will select maximum area on the image and then resize it to output resolution."),
      '#default_value' => $input!==FALSE ? $input['select_maximum_area'] : true,
    );

    $form['crop_widget_settings']['enforce_ratio'] = array(
      '#type'=>'checkbox',
      '#title'=>t('Enforce crop box ratio'),
      '#description'=> t("Check this to force the ratio of the output on the crop box. NOTE: If you leave this unchecked but enforce an output resolution, the final image might be distorted."),
      '#default_value' => $input!==FALSE ? $input['select_maximum_area'] : false,
    );

    $form['crop_widget_settings']['custom_ratio'] = array(
      '#type'=>'fieldset',
      '#title'=>t('Custom ratio'),
      '#attributes'=> array(
          'class' => array('group-box')
        ),
        '#states' => array(
          'visible' => array(
             array('input[name="enforce_ratio"]' => array('checked' => TRUE)),
          ),
        ),
      );
    $form['crop_widget_settings']['custom_ratio']['custom_ratio_x'] = array(
      '#type'=>'textfield',
      '#title'=>t('Width'),
      '#size'=>'6',
      '#field_suffix'=>'&nbsp;:&nbsp;&nbsp;',
    );
    $form['crop_widget_settings']['custom_ratio']['custom_ratio_y'] = array(
      '#type'=>'textfield',
      '#title'=>t('Height'),
      '#size'=>'6',
    );
    $form['crop_widget_settings']['custom_ratio']['desc'] = array(
      '#type'=>'markup',
      '#markup' =>'<div class="clearfix"></div>'.t('Ratio for crop area. For example 4:3 or 16:9 as width to height.'),
    );

    $form['crop_widget_settings']['enforce_minimum'] = array(
      '#type'=>'checkbox',
      '#title'=> t('Enforce minimum crop size based on the output size'),
      '#description'=> t("Check this to force a minimum cropping selection equal to the output size. NOTE: If you leave this unchecked you might get zoomed pixels if the cropping area is smaller than the output resolution."),
      '#default_value' => ($input!==FALSE && isset($input['enforce_minimum'])) ? $input['enforce_minimum'] : false,
    );

    $form['crop_widget_settings']['croparea'] = array(
      '#type'=>'fieldset',
      '#title'=> t('The resolution of the cropping area'),
      '#attributes'=> array(
        'class' => array('group-box')
      ),
    );
    $form['crop_widget_settings']['croparea']['croparea_x'] = array(
      '#type'=> 'textfield',
      '#title'=> t('Width'),
      '#size'=> '6',
      '#field_suffix'=>'&nbsp;×&nbsp;&nbsp;'

    );
    $form['crop_widget_settings']['croparea']['croparea_y'] = array(
      '#type'=>'textfield',
      '#title'=>t('Height'),
      '#size'=>'6',
      '#field_suffix'=>'px',
    );
    $form['crop_widget_settings']['croparea']['desc'] = array(
      '#type'=>'markup',
      '#markup' =>'<div class="clearfix"></div>'.t('The resolution of the area used for the cropping of the image. Image will displayed at this resolution for cropping. Use WIDTHxHEIGHT format, empty or zero values are permitted, e.g. 500px will limit crop box to 500 pixels width.'),
    );

    $form['crop_widget_settings']['gif_processing'] = array(
      '#type'=>'radios',
      '#title'=>t('Gif files processing'),
      '#options'=>array(
        'convert'=>t('Convert into jpeg and resize'),
        'skip'=>t('Do not resize'),
      ),
    '#default_value' => ($input==FALSE ? 'convert' : $input['gif_processing']),

    );
    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Run Process'),
    );

    return $form;
  }
  /**
   * Validation handler for admin settings form.
   */
  function admin_settings_validate($form, $form_state) {
    $form_state['rebuild'] = TRUE;

    if (empty($form_state['input']['field_to_convert'])) {
      form_set_error('field_to_convert', t('Available Image Field(s) has to have an field selected'));
      return;
    }
    $result = db_query('select * from {field_config} where `field_name` = :fn and `deleted` = 0', array(':fn' => $form_state['input']['field_to_convert']))->fetchAssoc();
    if ($result === FALSE) {
      form_set_error('field_to_convert', t('Invalid field. Not found in @table.', array('@table' => 'Available Image Field(s)')));
    }
  }

  function admin_settings_submit($form, &$form_state) {
    $form_state['rebuild'] = TRUE;

    $button = $form_state['values']['action_type'];
    $convert = FALSE;
    $unconvert = FALSE;
    if ($button == 'convert') {
      $convert = TRUE;
    } elseif ($button == 'unconvert') {
      $unconvert = TRUE;
    } elseif ($button != 'analyze') {
      drupal_set_message('Submit form state wrong: ' . serialize($form_state), 'error');
      return;
    }
    $field_name = $form_state['input']['field_to_convert'];
    $field_config_rows = db_query('select * from {field_config}'
      . ' where `field_name` = :fn and `deleted` = 0', array(':fn' => $field_name))->fetchAll();
    if (count($field_config_rows) == 0) {
      drupal_set_message($field_name . ' not found in field_config.', 'warning');
      return;
    }
    if (count($field_config_rows) > 1) {
      drupal_set_message($field_name . ' found more than once, ' . count($field_config_rows) . ' times,'
        . ' in field_config. Please report the error.',
        'error');
      return;
    }
    $config = reset($field_config_rows);
    $config->data = unserialize($config->data);
    //drupal_set_message('field_config row: ' . serialize($config));
    // $config is documented in field_config row.txt.
    drupal_set_message('Found "' . $field_name . '" in "field_config"  with id ' . $config->id . '.  <br />Type: ' . $config->type . ', module: ' . $config->module, 'notice');
    $field_config_instance_rows = array();
    $bundles = array();
    $message = 'Found ' . $field_name . ' in the following rows in "field_config_instance": ';
    $query = db_select('field_config_instance', 'i')
      ->fields('i')
      ->condition('field_name', $field_name,'=')
      ->condition('deleted', 0,'=')
      ->execute();
    foreach ($query as $row) {
      $row->data = unserialize($row->data);
      $field_config_instance_rows[$row->id] = $row;
      $message .= '<br />id: ' . $row->id . ', field id: ' . $row->field_id . ', entity type: ' . $row->entity_type . ', bundle: ' . $row->bundle;
      $bundles[$row->bundle] = $row->bundle;
      if (isset($row->data['label'])) {
        $message .= ', label: ' . $row->data['label'];
      }
      if (isset($row->data['widget']['module'])) {
        $message .= ', widget module: ' . $row->data['widget']['module'];
      }
    }
    drupal_set_message($message);
    foreach ($this->table_groups as $group => $field_load_name) {
      $table_details = $config->data['storage']['details']['sql'][$field_load_name];
      $table_keys = array_keys($table_details);
      $table = reset($table_keys);
      foreach ($bundles as $bundle) {
        $count = db_select($table, 't')->fields('t')->condition('bundle', $bundle, '=')->execute()->rowCount();
        drupal_set_message($count . ' rows for bundle ' . $bundle . ' in ' . $table . '.');
      }
    }
    if ($convert) {
      return $this->admin_settings_convert($form, $form_state, $config, $field_config_instance_rows);
    }
    if ($unconvert) {
      return $this->admin_settings_unconvert($form, $form_state, $config, $field_config_instance_rows);
    }
  }

  function admin_settings_convert($form, &$form_state, $config, $field_config_instance_rows) {
    $values = $form_state['values'];

    $field_name = $config->field_name;
    if ($config->type == 'imagefield_crop') {
      drupal_set_message($field_name . ' already converted from "image" to ' . $config->type . ' in field_config.', 'warning');
      return;
    }
    if ($config->type != 'image') {
      drupal_set_message($field_name . ' is not an "image" field in "field_config".', 'warning');
      return;
    }
    drupal_set_message('Converting ' . $field_name);
    /*
     * Change field_config.
     */
    if (isset($config->data['storage']['details']['sql'])) {
      if (isset($config->data['storage']['details']['sql']['FIELD_LOAD_CURRENT']['field_data_' . $field_name])) {
        $current = & $config->data['storage']['details']['sql']['FIELD_LOAD_CURRENT']['field_data_' . $field_name];
        if (isset($current['height'])) {
          $current['cropbox_height'] = $field_name . '_cropbox_height';
          unset($current['height']);
        }
        if (isset($current['width'])) {
          $current['cropbox_width'] = $field_name . '_cropbox_width';
          unset($current['width']);
        }
        $current['cropbox_x'] = $field_name . '_cropbox_x';
        $current['cropbox_y'] = $field_name . '_cropbox_y';
      }
      if (isset($config->data['storage']['details']['sql']['FIELD_LOAD_REVISION']['field_revision_' . $field_name])) {
        $revision = & $config->data['storage']['details']['sql']['FIELD_LOAD_REVISION']['field_revision_' . $field_name];
        if (isset($revision['height'])) {
          $revision['cropbox_height'] = $field_name . '_cropbox_height';
          unset($revision['height']);
        }
        if (isset($revision['width'])) {
          $revision['cropbox_width'] = $field_name . '_cropbox_width';
          unset($revision['width']);
        }
        $revision['cropbox_x'] = $field_name . '_cropbox_x';
        $revision['cropbox_y'] = $field_name . '_cropbox_y';
      }
    }
    $config->module = 'imagefield_crop';
    $config->type = 'imagefield_crop';
    $this->field_config_update($config);
    /*
     * Change field_config_instance.
     */
    foreach ($field_config_instance_rows as $instance_row) {
      drupal_set_message('Converting ' . $field_name . ' with id ' . $instance_row->id . ' in "field_config_instance".');
      //drupal_set_message('field_config_instance' . ' row: ' . serialize($config));
      //drupal_set_message('field_config_instance' . '.data before: ' . serialize($instance_row->data));

      $imagefield_crop_widget = $this->imagefield_crop_widget;
      $imagefield_crop_widget['settings']['progress_indicator'] = $values['progress_indicator'];
      $imagefield_crop_widget['settings']['preview_image_style'] = $values['preview_image_style'];
      $imagefield_crop_widget['settings']['collapsible'] = $values['collapsible'];
      $imagefield_crop_widget['settings']['resolution'] = $values['resolution_x'].'x'.$values['resolution_y']; // needs validition
      $imagefield_crop_widget['settings']['validate_resolution'] = $values['validate_resolution'];
      $imagefield_crop_widget['settings']['enforce_ratio'] = $values['enforce_ratio'];
      $imagefield_crop_widget['settings']['custom_ratio'] = $values['custom_ratio_x'].':'. $values['custom_ratio_y'];  // needs validition
      $imagefield_crop_widget['settings']['enforce_minimum'] = $values['enforce_minimum'];
      $imagefield_crop_widget['settings']['croparea'] = $values['croparea_x'].'x'.$values['croparea_x']; // needs validition
      $imagefield_crop_widget['settings']['gif_processing'] = $values['gif_processing'];
      $imagefield_crop_widget['settings']['select_maximum_area'] = $values['select_maximum_area'];

      $instance_row->data['widget'] = $imagefield_crop_widget;
      if (isset($instance_row->data['settings']['default_image'])) {
        unset($instance_row->data['settings']['default_image']);
      }
      if (isset($instance_row->data['settings']['mediafront'])) {
        unset($instance_row->data['settings']['mediafront']);
      }
      if (isset($instance_row->data['display']['weight'])) {
        $instance_row->data['display']['default']['weight'] = $instance_row->data['display']['weight'];
        unset($instance_row->data['display']['weight']);
      }
      if (isset($instance_row->data['display']['teaser'])) {
        unset($instance_row->data['display']['teaser']);
      }
      if (isset($instance_row->data['required'])) {
        $instance_row->data['required'] = (boolean) $instance_row->data['required'];
      }
      //drupal_set_message('field_config_instance' . '.data after: ' . serialize($instance_row->data));
      $this->field_config_instance_update($instance_row);
    }
    foreach ($this->table_groups as $group => $field_load_name) {
      $field_prefix = 'field_' . $group . '_';
      $this->alter_table_add_column($field_prefix, $field_name, $field_name . '_cropbox_x',"int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'X coordinate of left top point of cropping area.'");
      $this->alter_table_add_column($field_prefix, $field_name, $field_name . '_cropbox_y', "int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Y coordinate of left top point of cropping area.'");
      $this->alter_table_change_column($field_prefix, $field_name, $field_name . '_height', $field_name . '_cropbox_height', "int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Height value of cropping area.'");
      $this->alter_table_change_column($field_prefix, $field_name, $field_name . '_width', $field_name . '_cropbox_width', "int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Width value of cropping area.'");
    }
    cache_clear_all();
    field_cache_clear();
  }

  function admin_settings_unconvert($form, &$form_state, $config, $field_config_instance_rows) {
    $values = $form_state['values'];
    $field_name = $config->field_name;
    if ($config->type == 'image') {
      drupal_set_message($field_name . ' already converted from "imagefield_crop" to ' . $config->type . ' in "field_config".', 'warning');
      return;
    }
    if ($config->type !='imagefield_crop') {
      drupal_set_message($field_name . ' is not an "imagefield_crop" field in "field_config".', 'warning');
      return;
    }
    drupal_set_message('Unconverting ' . $field_name);
    /*
     * Change field_config.
     */
    if (isset($config->data['storage']['details']['sql'])) {
      if (isset($config->data['storage']['details']['sql']['FIELD_LOAD_CURRENT']['field_data_' . $field_name])) {
        $current = & $config->data['storage']['details']['sql']['FIELD_LOAD_CURRENT']['field_data_' . $field_name];
        if (isset($current['cropbox_height'])) {
          unset($current['cropbox_height']);
          $current['height'] = $field_name . '_height';
        }
        if (isset($current['cropbox_width'])) {
          unset($current['cropbox_width']);
          $current['width'] = $field_name . '_width';
        }
        unset($current['cropbox_x']);
        unset($current['cropbox_y']);
      }
      if (isset($config->data['storage']['details']['sql']['FIELD_LOAD_REVISION']['field_revision_' . $field_name])) {
        $revision = & $config->data['storage']['details']['sql']['FIELD_LOAD_REVISION']['field_revision_' . $field_name];
        if (isset($revision['cropbox_height'])) {
          unset($revision['cropbox_height']);
          $revision['height'] = $field_name . '_height';
        }
        if (isset($revision['cropbox_width'])) {
          unset($revision['cropbox_width']);
          $revision['width'] = $field_name . '_width';
        }
        unset($revision['cropbox_x']);
        unset($revision['cropbox_y']);
      }
    }
    $config->module = 'image';
    $config->type = 'image';
    $this->field_config_update($config);
    /*
     * Change field_config_instance.
     */
    foreach ($field_config_instance_rows as $instance_row) {
      drupal_set_message('Unconverting ' . $field_name . ' with id ' . $instance_row->id
        . ' in ' . 'field_config_instance' . '.');
      //drupal_set_message('field_config_instance' . ' row: ' . serialize($config));
      //drupal_set_message('field_config_instance' . '.data before: ' . serialize($instance_row->data));

      $image_widget = $this->image_widget;
      $image_widget['settings']['progress_indicator'] = $values['progress_indicator'];
      $image_widget['settings']['preview_image_style'] = $values['preview_image_style'];

      $instance_row->data['widget'] = $image_widget;
      if (!isset($instance_row->data['settings']['default_image'])) {
        $instance_row->data['settings']['default_image'] = 0;
      }
      if (!isset($instance_row->data['settings']['mediafront'])) {
        $instance_row->data['settings']['mediafront'] = $this->mediafront;
      }
      if (!isset($instance_row->data['display']['weight']) and isset($instance_row->data['display']['default']['weight'])) {
        $instance_row->data['display']['weight'] = $instance_row->data['display']['default']['weight'];
        unset($instance_row->data['display']['default']['weight']);
      }
      if (!isset($instance_row->data['display']['teaser'])) {
        $instance_row->data['display']['teaser'] = $this->image_teaser;
      }
      if (isset($instance_row->data['required'])) {
        $instance_row->data['required'] = (integer) $instance_row->data['required'];
      }
      //drupal_set_message('field_config_instance' . '.data after: ' . serialize($instance_row->data));
      $this->field_config_instance_update($instance_row);
    }
    foreach ($this->table_groups as $group => $field_load_name) {
      $field_prefix = 'field_' . $group . '_';
      $this->alter_table_delete_column($field_prefix, $field_name, $field_name . '_cropbox_x');
      $this->alter_table_delete_column($field_prefix, $field_name, $field_name . '_cropbox_y');
      $this->alter_table_change_column($field_prefix, $field_name, $field_name . '_cropbox_height', $field_name . '_height', "int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Height value of image area.'");
      $this->alter_table_change_column($field_prefix, $field_name, $field_name . '_cropbox_width', $field_name . '_width', "int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'Width value of image area.'");
    }
    cache_clear_all();
    field_cache_clear();
  }

  function alter_table_add_column($field_prefix, $field_name, $column, $attributes) {
    $result = db_query('alter table {' . $field_prefix . $field_name . '} add column `' . $column . '` ' . $attributes);
    if ($result === FALSE) {
      drupal_set_message('Alter failed for ' . $field_prefix . $field_name . ', add column ' . $column . '.', 'error');
      return FALSE;
    }
    drupal_set_message('Altered ' . $field_prefix . $field_name . ', add column ' . $column . '.');
    return TRUE;
  }

  function alter_table_change_column($field_prefix, $field_name, $old_column, $new_column, $attributes) {
    $result = db_query('alter table {' . $field_prefix . $field_name . '} change column `' . $old_column . '` `' . $new_column . '` ' . $attributes);
    if ($result === FALSE) {
      drupal_set_message('Alter failed for ' . $field_prefix . $field_name . ', change column from ' . $old_column . ' to ' . $new_column . '.', 'error');
      return FALSE;
    }
    drupal_set_message('Altered ' . $field_prefix . $field_name . ', change column from ' . $old_column . ' to ' . $new_column . '.');
    return TRUE;
  }

  function alter_table_delete_column($field_prefix, $field_name, $column) {
    $result = db_query('alter table {' . $field_prefix . $field_name . '} drop `' . $column . '`');
    if ($result === FALSE) {
      drupal_set_message('Alter failed for ' . $field_prefix . $field_name . ', delete column ' . $column . '.', 'error');
      return FALSE;
    }
    drupal_set_message('Altered ' . $field_prefix . $field_name . ', delete column ' . $column . '.');
    return TRUE;
  }

  function field_config_update($config) {
    $result = db_update('field_config')
      ->fields(array(
        'module' => $config->module,
        'type' => $config->type,
        'data' => serialize($config->data),
      ))
      ->condition('field_name', $config->field_name,'=')
      ->condition('id', $config->id,'=')
      ->execute();
    if ($result === FALSE) {
      $text = 'failed';
      $status = 'error';
    } else {
      $text = 'worked';
      $status = 'status';
      $result = TRUE;
    }
    drupal_set_message('Update of "field_config" ' . $text . ' for ' . $config->field_name . ' with id ' . $config->id . '.', $status);
    return $result;
  }

  function field_config_instance_update($instance_row) {
    // field_config_instance.id is the unique identifier in field_config_instance.
    $result = db_update('field_config_instance')
      ->fields(array('data' => serialize($instance_row->data)))
      ->condition('id', $instance_row->id,'=')
      ->execute();
    if ($result === FALSE) {
      $text = 'failed';
      $status = 'error';
    } else {
      $text = 'worked';
      $status = 'status';
      $result = TRUE;
    }
    drupal_set_message('Update of "field_config_instance" ' . $text . ' for ' . $instance_row->field_name . ' with id ' . $instance_row->id . ', entity_type ' . $instance_row->entity_type . ', bundle ' . $instance_row->bundle . '.', $status);
    return $result;
  }

}
